@@ -54,7 +54,7 @@ private:
     // md4qt
     MD::Parser<MD::QStringTrait> m_md4qtParser;
     int m_extendedSyntaxCount = 0;
-    const int m_id = MD::TextPlugin::UserDefinedPluginID + 1;
+    const int m_id = static_cast<int>(MD::TextPlugin::UserDefined) + 1;
 
     // Data
     QStringList m_testingLines = {
@@ -71,7 +71,7 @@ private:
         QStringLiteral("\\==Untouched=="),
         QStringLiteral("*==Unaffected*=="),
         QStringLiteral("^Unaffected\n^"),
-        QStringLiteral("New style mix-===-==-==-=="),
+        QStringLiteral("New style mix--===--==--==--=="),
         QStringLiteral("Multi__*line*__ ==*mix*== --of--\n^new^ --====and==-- original"),
         QStringLiteral("==*Cancelling part of== **original style***"),
         QStringLiteral("# ==*Cancelling part of== **original style in title***"),
@@ -97,7 +97,7 @@ void ExtendedSyntaxTest::addExtendedSyntaxs()
 {
     static const QList<QStringList> extendedSyntaxsList = {
         {QStringLiteral("=="), QStringLiteral("<mark>"), QStringLiteral("</mark>")}, // Highlight
-        {QStringLiteral("-"), QStringLiteral("<sub>"), QStringLiteral("</sub>")}, // Subscript
+        {QStringLiteral("--"), QStringLiteral("<sub>"), QStringLiteral("</sub>")}, // Subscript
         {QStringLiteral("^"), QStringLiteral("<sup>"), QStringLiteral("</sup>")}, // Superscript
     };
 
@@ -105,14 +105,14 @@ void ExtendedSyntaxTest::addExtendedSyntaxs()
         const long long int opts = MD::TextOption::StrikethroughText << (m_extendedSyntaxCount + 1);
 
         const QStringList options = {details[0], QString::number(opts)};
-        m_md4qtParser.addTextPlugin(m_id + m_extendedSyntaxCount, ExtendedSyntaxMaker::extendedSyntaxHelperFunc, true, options);
+        m_md4qtParser.addTextPlugin(static_cast<MD::TextPlugin>(m_id + m_extendedSyntaxCount), ExtendedSyntaxMaker::extendedSyntaxHelperFunc, true, options);
         ++m_extendedSyntaxCount;
     }
 }
 
 void ExtendedSyntaxTest::initTestCase()
 {
-    m_md4qtParser.addTextPlugin(1024, md4qtDataCleaner::dataCleaningFunc, false, {});
+    m_md4qtParser.addTextPlugin(md4qtDataCleaner::dataCleanerId, md4qtDataCleaner::dataCleaningFunc, false, {});
     addExtendedSyntaxs();
     makeTexts();
 }
@@ -613,7 +613,7 @@ void ExtendedSyntaxTest::unaffected2()
 };
 
 /*
-New style mix-===-==-==-==
+New style mix--===--==--==--==
 */
 void ExtendedSyntaxTest::newStyleMix()
 {
@@ -639,13 +639,13 @@ void ExtendedSyntaxTest::newStyleMix()
     const auto &openStyles2 = item2->openStyles();
     QCOMPARE_EQ(openStyles2.length(), 1);
     QCOMPARE_EQ(openStyles2[0].startColumn(), 13);
-    QCOMPARE_EQ(openStyles2[0].endColumn(), 13);
+    QCOMPARE_EQ(openStyles2[0].endColumn(), 14);
     QCOMPARE_EQ(openStyles2[0].startLine(), 0);
     QCOMPARE_EQ(openStyles2[0].endLine(), 0);
     const auto &closeStyles2 = item2->closeStyles();
     QCOMPARE_EQ(closeStyles2.length(), 1);
-    QCOMPARE_EQ(closeStyles2[0].startColumn(), 17);
-    QCOMPARE_EQ(closeStyles2[0].endColumn(), 17);
+    QCOMPARE_EQ(closeStyles2[0].startColumn(), 18);
+    QCOMPARE_EQ(closeStyles2[0].endColumn(), 19);
     QCOMPARE_EQ(closeStyles2[0].startLine(), 0);
     QCOMPARE_EQ(closeStyles2[0].endLine(), 0);
     // !Check delims
@@ -662,14 +662,14 @@ void ExtendedSyntaxTest::newStyleMix()
     // Check delims
     const auto &openStyles4 = item4->openStyles();
     QCOMPARE_EQ(openStyles4.length(), 1);
-    QCOMPARE_EQ(openStyles4[0].startColumn(), 20);
-    QCOMPARE_EQ(openStyles4[0].endColumn(), 20);
+    QCOMPARE_EQ(openStyles4[0].startColumn(), 22);
+    QCOMPARE_EQ(openStyles4[0].endColumn(), 23);
     QCOMPARE_EQ(openStyles4[0].startLine(), 0);
     QCOMPARE_EQ(openStyles4[0].endLine(), 0);
     const auto &closeStyles4 = item4->closeStyles();
     QCOMPARE_EQ(closeStyles4.length(), 1);
-    QCOMPARE_EQ(closeStyles4[0].startColumn(), 23);
-    QCOMPARE_EQ(closeStyles4[0].endColumn(), 23);
+    QCOMPARE_EQ(closeStyles4[0].startColumn(), 26);
+    QCOMPARE_EQ(closeStyles4[0].endColumn(), 27);
     QCOMPARE_EQ(closeStyles4[0].startLine(), 0);
     QCOMPARE_EQ(closeStyles4[0].endLine(), 0);
     // !Check delims
@@ -759,25 +759,18 @@ void ExtendedSyntaxTest::multiLineMix()
     const auto item5 = std::static_pointer_cast<MD::Text<MD::QStringTrait>>(paragraph->getItemAt(4));
     // Check delims
     const auto &openStyles5 = item5->openStyles();
-    QCOMPARE_EQ(openStyles5.length(), 2);
+    QCOMPARE_EQ(openStyles5.length(), 1);
     QCOMPARE_EQ(openStyles5[0].startColumn(), 26);
-    QCOMPARE_EQ(openStyles5[0].endColumn(), 26);
+    QCOMPARE_EQ(openStyles5[0].endColumn(), 27);
     QCOMPARE_EQ(openStyles5[0].startLine(), 0);
     QCOMPARE_EQ(openStyles5[0].endLine(), 0);
-    QCOMPARE_EQ(openStyles5[1].startColumn(), 27);
-    QCOMPARE_EQ(openStyles5[1].endColumn(), 27);
-    QCOMPARE_EQ(openStyles5[1].startLine(), 0);
-    QCOMPARE_EQ(openStyles5[1].endLine(), 0);
     const auto &closeStyles5 = item5->closeStyles();
-    QCOMPARE_EQ(closeStyles5.length(), 2);
+    QCOMPARE_EQ(closeStyles5.length(), 1);
     QCOMPARE_EQ(closeStyles5[0].startColumn(), 30);
-    QCOMPARE_EQ(closeStyles5[0].endColumn(), 30);
+    QCOMPARE_EQ(closeStyles5[0].endColumn(), 31);
     QCOMPARE_EQ(closeStyles5[0].startLine(), 0);
     QCOMPARE_EQ(closeStyles5[0].endLine(), 0);
-    QCOMPARE_EQ(closeStyles5[1].startColumn(), 31);
-    QCOMPARE_EQ(closeStyles5[1].endColumn(), 31);
-    QCOMPARE_EQ(closeStyles5[1].startLine(), 0);
-    QCOMPARE_EQ(closeStyles5[1].endLine(), 0);
+
     // !Check delims
     QCOMPARE_EQ(item5->opts(), 16);
     QCOMPARE(item5->text(), QStringLiteral("of"));
@@ -803,15 +796,11 @@ void ExtendedSyntaxTest::multiLineMix()
     const auto item7 = std::static_pointer_cast<MD::Text<MD::QStringTrait>>(paragraph->getItemAt(6));
     // Check delims
     const auto &openStyles7 = item7->openStyles();
-    QCOMPARE_EQ(openStyles7.length(), 2);
+    QCOMPARE_EQ(openStyles7.length(), 1);
     QCOMPARE_EQ(openStyles7[0].startColumn(), 6);
-    QCOMPARE_EQ(openStyles7[0].endColumn(), 6);
+    QCOMPARE_EQ(openStyles7[0].endColumn(), 7);
     QCOMPARE_EQ(openStyles7[0].startLine(), 1);
     QCOMPARE_EQ(openStyles7[0].endLine(), 1);
-    QCOMPARE_EQ(openStyles7[1].startColumn(), 7);
-    QCOMPARE_EQ(openStyles7[1].endColumn(), 7);
-    QCOMPARE_EQ(openStyles7[1].startLine(), 1);
-    QCOMPARE_EQ(openStyles7[1].endLine(), 1);
     QCOMPARE_EQ(item7->closeStyles().length(), 0);
     // !Check delims
     QCOMPARE_EQ(item7->opts(), 16);
@@ -826,19 +815,16 @@ void ExtendedSyntaxTest::multiLineMix()
     QCOMPARE_EQ(openStyles8[0].startLine(), 1);
     QCOMPARE_EQ(openStyles8[0].endLine(), 1);
     const auto &closeStyles8 = item8->closeStyles();
-    QCOMPARE_EQ(closeStyles8.length(), 3);
+    QCOMPARE_EQ(closeStyles8.length(), 2);
     QCOMPARE_EQ(closeStyles8[0].startColumn(), 15);
     QCOMPARE_EQ(closeStyles8[0].endColumn(), 16);
     QCOMPARE_EQ(closeStyles8[0].startLine(), 1);
     QCOMPARE_EQ(closeStyles8[0].endLine(), 1);
     QCOMPARE_EQ(closeStyles8[1].startColumn(), 17);
-    QCOMPARE_EQ(closeStyles8[1].endColumn(), 17);
+    QCOMPARE_EQ(closeStyles8[1].endColumn(), 18);
     QCOMPARE_EQ(closeStyles8[1].startLine(), 1);
     QCOMPARE_EQ(closeStyles8[1].endLine(), 1);
-    QCOMPARE_EQ(closeStyles8[2].startColumn(), 18);
-    QCOMPARE_EQ(closeStyles8[2].endColumn(), 18);
-    QCOMPARE_EQ(closeStyles8[2].startLine(), 1);
-    QCOMPARE_EQ(closeStyles8[2].endLine(), 1);
+
     // !Check delims
     QCOMPARE_EQ(item8->opts(), 24);
     QCOMPARE(item8->text(), QStringLiteral("and"));
