@@ -304,11 +304,11 @@ void handleReturnPressed(const MdEditor::EditorHandler *editor, const MD::ListIt
     }
 
     QString str = modifier == keyModif::ShiftModifer ? QStringLiteral("<br>\n") : QStringLiteral("\n");
-    if (listItem) {
+    if (listItem && listItem->startColumn() < cursor.positionInBlock()) {
         if (listItem->isEmpty()) {
             QTextBlock block = cursor.block();
 
-            removeText(cursor, block.position(), block.position() + block.length());
+            removeText(cursor, block.position(), block.position() + block.length() - 1);
         } else {
             const int indent = listItem->delim().startColumn();
             str += useSpaceForTab ? QStringLiteral(" ").repeated(indent) : QStringLiteral("\t").repeated(indent);
