@@ -16,7 +16,7 @@ Parser::Parser()
 {
     connect(this, &Parser::newData, this, &Parser::onParse, Qt::QueuedConnection);
     connectPlugins();
-    m_md4qtParser.addTextPlugin(1024, md4qtDataCleaner::dataCleaningFunc, false, {});
+    m_md4qtParser.addTextPlugin(md4qtDataCleaner::dataCleanerId, md4qtDataCleaner::dataCleaningFunc, false, {});
 }
 
 // Connections
@@ -37,15 +37,15 @@ void Parser::connectPlugins()
 // =======
 void Parser::addExtendedSyntax(const QStringList &details)
 {
-    m_md4qtParser.addTextPlugin(details.last().toInt(), ExtendedSyntaxMaker::extendedSyntaxHelperFunc, true, details);
+    m_md4qtParser.addTextPlugin(static_cast<MD::TextPlugin>(details.last().toInt()), ExtendedSyntaxMaker::extendedSyntaxHelperFunc, true, details);
 }
 
 void Parser::addRemovePlugin(const int pluginId, const bool add)
 {
     if (add) {
-        m_md4qtParser.addTextPlugin(pluginId, m_kleverPlugins.at(pluginId), true, {});
+        m_md4qtParser.addTextPlugin(static_cast<MD::TextPlugin>(pluginId), m_kleverPlugins.at(pluginId), true, {});
     } else {
-        m_md4qtParser.removeTextPlugin(pluginId);
+        m_md4qtParser.removeTextPlugin(static_cast<MD::TextPlugin>(pluginId));
     }
 }
 // !Setters
@@ -64,11 +64,12 @@ void Parser::quickEmojiEnabledChanged()
 // !KleverNotes slots
 
 // markdown-tools editor slots
-void Parser::onData(const QString &md, const QString &notePath, unsigned long long int counter)
+void Parser::onData(const QString &md, const QString &noteDir, const QString &noteName, unsigned long long int counter)
 {
     m_data.clear();
     m_data.push_back(md);
-    m_notePath = notePath;
+    m_noteDir = noteDir;
+    m_noteName = noteName;
     m_counter = counter;
 
     Q_EMIT newData();
@@ -79,7 +80,7 @@ void Parser::onParse()
     if (!m_data.isEmpty()) {
         QTextStream stream(&m_data.back());
 
-        const auto doc = m_md4qtParser.parse(stream, m_notePath, QStringLiteral("note.md"), false);
+        const auto doc = m_md4qtParser.parse(stream, m_noteDir, m_noteName, false);
 
         m_data.clear();
 
