@@ -89,7 +89,7 @@ TreeItem::TreeItem(const QString &path, NoteTreeModel *model, TreeItem *parentIt
 
         const int nameIdx = m_tempChildrenNames.indexOf(node->m_name);
         if (nameIdx != -1) {
-            node->place = nameIdx;
+            // node->place = nameIdx;
             const auto refChild = m_tempChildrenInfo[nameIdx].toObject();
 
             setFromMetadata(refChild, QStringLiteral("icon"), node->m_icon);
@@ -99,15 +99,10 @@ TreeItem::TreeItem(const QString &path, NoteTreeModel *model, TreeItem *parentIt
         appendChild(std::move(node));
     }
 
-    m_tempChildrenNames = {}; // Node need to keep it
-    m_tempChildrenInfo = {}; // Node need to keep it
-    std::sort(m_children.begin(), m_children.end(), [](auto const &a, auto const &b) {
-        if (a->place == -1)
-            return false;
-        if (b->place == -1)
-            return true;
-        return a->place < b->place;
-    });
+    m_tempChildrenNames = {}; // No need to keep it
+    m_tempChildrenInfo = {}; // No need to keep it
+
+    orderChildren();
 
     saveMetaData();
 }
@@ -164,7 +159,37 @@ void TreeItem::saveMetaData()
     DocumentHandler::saveJson(metadata, metadataPath);
 }
 
-void TreeItem::appendChild(std::unique_ptr<TreeItem> &&item)
+int TreeItem::compare(const QString &name, bool isNote) const
+{
+    if (m_isNote != isNote)
+        return !m_isNote ? -1 : 1;
+
+    return QString::compare(m_name, name, Qt::CaseSensitive);
+}
+
+int TreeItem::compare(const std::unique_ptr<TreeItem> &other) const
+{
+    return compare(other->m_name, other->m_isNote);
+}
+
+int TreeItem::getNewChildIndex(const QString &name, bool isNote) const
+{
+    int index = static_cast<int>(m_children.size());
+    for (int i = 0; i < index; i++) {
+        if (m_children[i]->compare(name, isNote) > 0)
+            return i;
+    }
+    return index;
+}
+
+void TreeItem::orderChildren()
+{
+    std::sort(m_children.begin(), m_children.end(), [](auto const &a, auto const &b) {
+        return a->compare(b) < 0;
+    });
+}
+
+void TreeItem::insertChild(std::unique_ptr<TreeItem> &&item, int index)
 {
     if (item->isNote() && m_model->noteMapEnabled()) {
         // very important to make a copy here !
@@ -178,7 +203,17 @@ void TreeItem::appendChild(std::unique_ptr<TreeItem> &&item)
     if (item->getParentItem() != this) {
         item->setParentItem(this);
     }
-    m_children.push_back(std::move(item));
+
+    if (index == static_cast<int>(m_children.size())) {
+        m_children.push_back(std::move(item));
+    } else {
+        m_children.insert(m_children.begin() + index, std::move(item));
+    }
+}
+
+void TreeItem::appendChild(std::unique_ptr<TreeItem> &&item)
+{
+    insertChild(std::move(item), m_children.size());
 }
 
 TreeItem *TreeItem::child(int row) const
